#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo cli run)
  #:declarative? #f
  #:use-module (config)
  #:use-module (config api)
  #:use-module (ice-9 filesystem)
  #:use-module (ordo core)
  #:use-module (ordo inventory)
  #:use-module (ordo logger)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-26)
  #:export (config handler))

(define (valid-tags? x)
  (or (null? x)
      (and (list? x) (every keyword? x))))

(define (parse-tags x)
  (map (compose symbol->keyword string->symbol)
       (if (list? x) x (list x))))

(define config
  (configuration
   (name 'run)
   (wanted '((keywords . (log-level)) directory))
   (keywords
    (list
     (setting
      (name 'inventory)
      (default "/dev/null")
      (example "examples/inventory.scm")
      (handler (cut expand-file-name <> #f #t))
      (test file-exists?)
      (synopsis "Inventory file"))
     (switch
      (name 'tag)
      (default (list))
      (test valid-tags?)
      (handler parse-tags)
      (merge-strategy cons)
      (synopsis "Limit operations to specified tag(s)"))))
   (arguments
    (list
     (argument
      (name 'workflow)
      (handler (cut expand-file-name <> #f #t))
      (example "examples/uptime.scm")
      (test file-exists?))))
   (synopsis "Run a workflow")))

(define (handler options)
  (load-inventory! (option-ref options 'inventory))
  (load (option-ref options '(workflow))))
